@@ -1,10 +1,10 @@
 /* main.c - main program and argument processing for cpio.
-   Copyright (C) 1990, 1991, 1992, 2001, 2003, 2004, 2005, 2006
-   Free Software Foundation, Inc.
+   Copyright (C) 1990, 1991, 1992, 2001, 2003, 2004, 2005, 2006,
+   2007 Free Software Foundation, Inc.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2, or (at your option)
+   the Free Software Foundation; either version 3, or (at your option)
    any later version.
 
    This program is distributed in the hope that it will be useful,
@@ -32,7 +32,7 @@
 #include <sys/stat.h>
 
 #ifdef HAVE_LOCALE_H
-#  include <locale.h>
+# include <locale.h>
 #endif
 
 #include "filetypes.h"
@@ -40,7 +40,7 @@
 #include "dstring.h"
 #include "extern.h"
 #include <rmt.h>
-#include <localedir.h>
+#include <rmt-command.h>
 
 enum cpio_options {
   NO_ABSOLUTE_FILENAMES_OPTION=256,
@@ -249,7 +249,7 @@ license ()
 	  "Copyright (C) 2004 Free Software Foundation, Inc.\n");
   printf (_("   GNU cpio is free software; you can redistribute it and/or modify\n"
     "   it under the terms of the GNU General Public License as published by\n"
-    "   the Free Software Foundation; either version 2 of the License, or\n"
+    "   the Free Software Foundation; either version 3 of the License, or\n"
     "   (at your option) any later version.\n"
     "\n"
     "   GNU cpio is distributed in the hope that it will be useful,\n"
@@ -273,6 +273,7 @@ warn_control (char *arg)
     { "none",       CPIO_WARN_ALL       },
     { "truncate",   CPIO_WARN_TRUNCATE  },
     { "all",        CPIO_WARN_ALL       },
+    { "interdir",   CPIO_WARN_INTERDIR  },
     { NULL }
   };
   struct warn_tab *wt;
@@ -667,7 +668,6 @@ process_args (int argc, char *argv[])
       CHECK_USAGE(retain_time_flag, "--preserve-modification-time",
 		  "--create");
       CHECK_USAGE(no_chown_flag, "--no-preserve-owner", "--create");
-      CHECK_USAGE(set_owner_flag||set_group_flag, "--owner", "--create");
       CHECK_USAGE(swap_bytes_flag, "--swap-bytes (--swap)", "--create");
       CHECK_USAGE(swap_halfwords_flag, "--swap-halfwords (--swap)",
 		  "--create");
@@ -691,8 +691,10 @@ process_args (int argc, char *argv[])
   else
     {
       /* Copy pass.  */
-      if (index != argc - 1)
+      if (index < argc - 1)
 	error (PAXEXIT_FAILURE, 0, _("Too many arguments"));
+      else if (index > argc - 1)
+	error (PAXEXIT_FAILURE, 0, _("Not enough arguments"));
 
       if (archive_format != arf_unknown)
 	error (PAXEXIT_FAILURE, 0, 
