@@ -1,9 +1,9 @@
 /* mt -- control magnetic tape drive operation
-   Copyright (C) 1991, 1992, 1995, 2001 Free Software Foundation, Inc.
+   Copyright (C) 1991, 1992, 1995, 2001, 2007 Free Software Foundation, Inc.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2, or (at your option)
+   the Free Software Foundation; either version 3, or (at your option)
    any later version.
 
    This program is distributed in the hope that it will be useful,
@@ -75,7 +75,7 @@
 #ifdef HAVE_LOCALE_H
 #  include <locale.h>
 #endif
-#include <localedir.h>
+#include <rmt-command.h>
 
 #if defined(HAVE_UNISTD_H)
 #include <unistd.h>
